local addon = LibStub("AceAddon-3.0"):GetAddon("Ouro Loot",--[[silent=]]true)
if not addon then return end
local plugin = addon:ConstrainedNewModule("HTML")
if not plugin then return end
local nametag = ...

local pairs, ipairs, tostring, tonumber = pairs, ipairs, tostring, tonumber
local tconcat, wipe = table.concat, table.wipe
local GUI = LibStub("AceGUI-3.0")

plugin.option_defaults = { profile = {
	['time'] = "strong",
	['instance'] = "strong",
	['boss'] = "em",
	['loot'] = [[<p><a href="http://www.wowhead.com/?item=$I">$N</a>$X - $T</p>]],
} }
local option_descriptions = {
	['time'] = [[Date Header]],
	['instance'] = [[Instance Header]],
	['boss'] = [[Boss Encounter Name]],
	['loot'] = [[Loot Entry]],
}

function plugin:OnEnable()
	self:default_OnEnable()
	self:Print[[WARNING!  Generating HTML is slow and sloppy and will cause your game client to freeze momentarily.  Do not panic.  Just get a beverage refill and think happy thoughts.]]

	local displayname = "HTML"
	local version = GetAddOnMetadata (nametag, "Version")
	if version then
		displayname = displayname .. " " .. version
	end

	addon:register_options_entry (plugin, --[[parent=]]nil, --[[text=]]nil,
	{
		type = 'group',
		handler = plugin,
		get = "GetOption",
		set = "SetOption",
		args = {
			-- This key is special; contents filled in by parent addon
			name = {
				name = displayname,
				type = 'description',
			},

			nothingyet = {
				name = [[All controlling variables to date are set on the HTML text tab directly, as you'll probably want to fiddle with them often for different situations.]],
				type = 'description',
				width = 'full',
				order = 10,
			},
		},
	})
end


local function mkwidgets (optiontable, tag)
	local label = GUI:Create("Label")
	label:SetRelativeWidth(0.30)
	label:SetText(option_descriptions[tag])
	label:SetFontObject(GameFontNormalLarge)
	label:SetColor(30/255,0xad/255,1)   -- same blue as "+" in help markup
	local editbox = GUI:Create("EditBox")
	editbox:SetRelativeWidth(0.66)
	--editbox:SetLabel(.....)
	editbox:SetText(optiontable[tag])
	editbox:SetCallback("OnEnterPressed", function(_e,event,value)
		optiontable[tag] = value
		_e.editbox:ClearFocus()
	end)
	return label, editbox
end

local function ShowEditor()
	local keep = CopyTable(plugin.opts)   -- hm

	local W = GUI:Create("Window")
	-- This is a horrible way to loop over an ellipsis.  Good thing it's
	-- called only rarely.
	for _,r in ipairs{W.frame:GetRegions()} do
		if r --and r.IsObjectType
		   and r:IsObjectType("Texture")
		   and r:GetTexture() == [[Interface\Tooltips\UI-Tooltip-Background]]
		then
			r:SetVertexColor(0,0,0,1)  -- try to make the background opaque
			break
		end
	end
	W:SetTitle("HTML Tags")
	W:SetLayout("List")
	W:SetStatusTable{width=850,height=400}
	W:EnableResize(false)
	W:SetCallback("OnClose", function(_w)
		addon:OpenMainDisplayToTab[[HTML]]
	end)

	local scrollcontainer = GUI:Create("SimpleGroup")
	scrollcontainer:SetFullWidth(true)
	--scrollcontainer:SetFullHeight(true)
	-- Calling setfullheight doesn't work; the "full height" at this point
	-- is the scrolling frame's default height of about 50 pixels.  Think of
	-- this as the maximum allowed height the scrolling frame is permitted
	-- to grow to; it will use less if needed.
	scrollcontainer:SetHeight(0.95 * W.status.height)
	scrollcontainer:SetLayout("Fill")
	W:AddChild(scrollcontainer)

	local scroll = GUI:Create("ScrollFrame")
	scroll:SetLayout("Flow")
	scrollcontainer:AddChild(scroll)

	local txt = GUI:Create("Label")
	txt:SetFullWidth(true)
	txt:SetFontObject(GameFontNormal)
	txt:SetText(([[
This is the template for loot entries, using the same token replacements as for forum markup:
$I    numeric item ID (capital-EYE, not lowercase-ELL)
$N    item name
$X    number in stack, or empty if only one
$T    "disposition" (player name, offspec, vault, etc)
]]):trim())
	scroll:AddChild(txt)
	scroll:AddChildren(mkwidgets(keep,'loot'))

	local spacer = GUI:Create("Spacer")
	spacer:SetFullWidth(true)
	spacer:SetHeight(20)

	txt = GUI:Create("Label")
	txt:SetFullWidth(true)
	txt:SetFontObject(GameFontNormal)
	txt:SetText(([[The rest of the options are single HTML tags which surround the corresponding kinds of output text.]]):trim())
	scroll:AddChildren(spacer,txt)
	scroll:AddChildren(mkwidgets(keep,'time'))
	scroll:AddChildren(mkwidgets(keep,'instance'))
	scroll:AddChildren(mkwidgets(keep,'boss'))

	spacer = GUI:Create("Spacer")
	spacer:SetFullWidth(true)
	spacer:SetHeight(20)
	local okay = GUI:Create("Button")
	okay:SetRelativeWidth(0.18)
	okay:SetText("Save & Close")
	okay:SetCallback("OnClick", function(_o)
		for k,v in pairs(keep) do -- don't change the opts pointer itself
			plugin.opts[k] = v
		end
		W:Hide()
	end)
	local cancel = GUI:Create("Button")
	cancel:SetRelativeWidth(0.28)
	cancel:SetText("Discard Changes & Close")
	cancel:SetCallback("OnClick", function(_c)
		W:Hide()
	end)
	scroll:AddChildren (spacer, okay, cancel)

	W:ApplyStatus()
	W:Show()
end


local function generator (_, loot, last_printed, generated, cache)
	local opts = plugin.opts

	for i = last_printed+1, #loot do
		local e = loot[i]

		if e.kind == 'loot' then
			-- see notes in corresponding code of text_tabs.lua
			local disp = e.disposition or e.person
			if disp == 'offspec' then
				disp = e.person .. " " .. 'offspec'
			elseif disp == 'gvault' then
				--disp = "guild vault (".. e.person .. ")"
				disp = [[guild vault]]
			end
			if e.extratext_byhand then
				disp = disp .. " -- " .. e.extratext
			end
			local t = opts.loot:gsub('%$I', e.id)
			                   :gsub('%$N', e.itemname)
			                   :gsub('%$X', e.count or "")
			                   :gsub('%$T', disp)

			cache[#cache+1] = t

		elseif e.kind == 'boss' and e.reason == 'kill' then
			-- first boss in an instance gets an instance tag, others get a blank line
			if generated.last_instance == e.instance then
				cache[#cache+1] = [[<br/>]]
			else
				cache[#cache+1] = ([[<br/><%s>%s</%s>]]):format (opts.instance,
					e.bossname, opts.instance)
				generated.last_instance = e.instance
			end
			cache[#cache+1] = ([[<%s>%s</%s><br/>]]):format (opts.boss, e.bossname, opts.boss)

		elseif e.kind == 'time' then
			cache[#cache+1] = ([[<%s>%s</%s><br/>]]):format (opts.time, e.startday.text, opts.time)

		end

	end
	return true
end

local function specials (_, editbox, container, mkbutton)
	local b = mkbutton("HTML Tags...",
		[[What markup tags to use.  Closing the tags editor reopens this window.]])
	b:SetFullWidth(true)
	b:SetCallback("OnClick", function(_b)
		addon.display:Hide()
		ShowEditor()
	end)
	container:AddChild(b)
end

plugin:register_text_generator ("html", [[HTML]], [[Raw HTML output]], generator, specials)

-- vim:noet
